<?php

namespace Drupal\gcal_entity\Form;

use Drupal\Core\Config\TypedConfigManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\gcal_entity\GcalSettingsService;
use Symfony\Component\DependencyInjection\ContainerInterface;

class GcalConfigForm extends ConfigFormBase {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The GcalSettingsService.
   *
   * @var \Drupal\gcal_entity\GcalSettingsService
   */
  protected $gcalSettings;

  /**
   * Constructs a GcalConfigForm object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\gcal_entity\GcalSettingsService $gcal_settings
   *   The GcalSettingsService.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory, TypedConfigManagerInterface $configTyped,
    \Drupal\gcal_entity\GcalSettingsService $gcal_settings
  ) {
    parent::__construct($config_factory, $configTyped);
    $this->configFactory = $config_factory;
    $this->gcalSettings = $gcal_settings;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var ConfigFactoryInterface $configFactory */
    $configFactory = $container->get('config.factory');
    /** @var TypedConfigManagerInterface $configTyped */
    $configTyped = $container->get('config.typed');

    /** @var GcalSettingsService $gcalSettingsService */
    $gcalSettingsService = $container->get('gcal_entity.settings');
    return new static(
      $configFactory,
      $configTyped,
      $gcalSettingsService
    );
  }

  protected function getEditableConfigNames(): array {
    return [
      'gcal_entity.settings',
    ];
  }

  public function getFormId(): string {
    return 'gcal_entity_admin_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config('gcal_entity.settings');
    $form['api_key'] = [
      '#type' => 'fieldset',
      '#title' => t('Google API Key'),
      '#open' => TRUE,
    ];

    $form['api_key']['googleapi'] = [
      '#type' => 'textfield',
      '#title' => t('Enter your key'),
      '#default_value' => $this->gcalSettings->getGoogleApiKey(),
      '#size' => 39,
      '#maxlength' => 39,
      '#description' => t('Key for server applications - <a href="https://console.cloud.google.com/apis/api/calendar-json.googleapis.com">https://console.cloud.google.com/apis/api/calendar-json.googleapis.com</a>'),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];
    $form['api_key']['description'] =[
      '#type' => 'item',
      '#markup' => t('Using the <strong>database</strong> to store the key is recommended.'),
    ];
    $form['api_key']['api_key_storage'] = [
      '#type' => 'radios',
      '#title' => t('Google API Key Storage'),
      '#options' => [
        'state' => t('Database'),
        'config' => t('Configuration'),
      ],
      '#default_value' => $this->gcalSettings->getApiStorageType(),
    ];
    $form['interface_settings'] = [
      '#type' => 'fieldset',
      '#title' => t('Interface settings'),
      '#open' => TRUE,
    ];
    $form['interface_settings']['start'] = [
      '#type' => 'textfield',
      '#title' => t('Event start'),
      '#default_value' => $config->get('start'),
      '#description' => t('Any events that are older than this are not displayed. For example "-1 day" or "yesterday" will include events that occured yesterday, where as "now" will only include events that have not yet started. You can use any string <a href="http://php.net/strtotime">strtotime</a> can parse.'),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['end'] = [
      '#type' => 'textfield',
      '#title' => t('Event end'),
      '#default_value' => $this->gcalSettings->getEventEnd(),
      '#description' => t('Any events newer than this are not displayed. For example, "+2 months" will display any events that occur in the next 2 months. You can use any string <a href="http://php.net/strtotime">strtotime</a> can parse.'),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];
    $form['interface_settings']['datelimit'] = [
      '#type' => 'number',
      '#title' => t('Dates to display'),
      '#default_value' => $this->gcalSettings->getDateLimit(),
      '#description' => t("The maximum number of unique dates to display."),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['maxevents'] = [
      '#type' => 'number',
      '#title' => t('Maximum number of events to fetch'),
      '#default_value' => $this->gcalSettings->getMaxEvents(),
      '#description' => t("The maximum number of events to fetch."),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['dateformat'] = [
      '#type' => 'textfield',
      '#title' => t('Date formatting'),
      '#default_value' => $this->gcalSettings->getDateFormat(),
      '#description' => t('Specify the date format.'),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['timeformat'] = [
      '#type' => 'textfield',
      '#title' => t('Time formatting'),
      '#default_value' => $this->gcalSettings->getTimeFormat(),
      '#description' => t('Specify the time format.'),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['noeventstext'] = [
      '#type' => 'textfield',
      '#title' => t('Text to display if no events are found'),
      '#default_value' => $this->gcalSettings->getNoEventsText(),
      '#description' => t('Leave blank to have the block hide when no events are found.'),
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['linktext'] = [
      '#type' => 'textfield',
      '#title' => t('Calendar link text'),
      '#default_value' => $this->gcalSettings->getLinkText(),
      '#description' => t('Text to display when linking to the Google Calendar event.'),
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['hangoutlinktext'] = [
      '#type' => 'textfield',
      '#title' => t('Calendar Meet link text'),
      '#default_value' => $this->gcalSettings->getHangoutLinkText(),
      '#description' => t('Text to display when linking to the Google Calendar event Hangout.'),
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['cachetime'] = [
      '#type' => 'number',
      '#title' => t('Amount of time to cache event data'),
      '#default_value' => $this->gcalSettings->getCacheTime(),
      '#description' => t("How long (in seconds) the module should hold onto the ICS data before re-requesting it from Google."),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
    ];

    $form['interface_settings']['timezone'] = [
      '#type' => 'select',
      '#title' => t('Timezone'),
      '#default_value' => $this->gcalSettings->getTimezone(),
      '#description' => t('The timezone identifier to be used for this calendar, as described in <a href="http://php.net/timezones">the PHP manual</a>.'),
      '#required' => TRUE,
      '#gcal_entity_setting' => TRUE,
      '#options' => array_combine(timezone_identifiers_list(), timezone_identifiers_list()),
    ];

    return parent::buildForm($form, $form_state);
  }



  /**
   * {@inheritdoc}
   */
  function submitForm(array &$form, FormStateInterface $form_state): void {
    $config_keys = [
      'start',
      'end',
      'datelimit',
      'maxevents',
      'dateformat',
      'timeformat',
      'noeventstext',
      'linktext',
      'hangoutlinktext',
      'cachetime',
      'timezone',
    ];

    $values = [];
    foreach ($config_keys as $key) {
      $values[$key] = $form_state->getValue($key);
    }
    // Set the values in the config.
    $results = $this->gcalSettings->setMultiple($values);
    // Set the api_key_storage value.
    $this->gcalSettings->setApiKeyStorageType($form_state->getValue('api_key_storage'));
    // Set the api_key value.
    $api_results =  $this->gcalSettings->setGoogleApiKey($form_state->getValue('googleapi'));
    // Return a report of what was changed.
    $this->messenger()->addStatus(t('<p>@changed</p><p>@api_key</p>', [
      '@changed' => $results,
      '@api_key' => $api_results,
    ]));


  }

}
